// Board.cpp: implementation of the CBoard class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Carrom.h"
#include "Board.h"
#include <Math.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// class Peg
//////////////////////////////////////////////////////////////////////

CPeg::PegInfo CPeg::piGeneral={6,1,-150};
CPeg::PegInfo CPeg::piStricker={8,1.6,-100};
CBrush CPeg::cbrPegBlack;
CBrush CPeg::cbrPegWhite;
CBrush CPeg::cbrPegRed;
CBrush CPeg::cbrPegStricker;

CPeg::CPeg(EPegTypes Type)
{
	PegType=Type;
	nPocketNo=1;
	Rest();
}
void CPeg::DrawPeg(CDC *dc)
{
	if (nPocketNo) return;
	CBrush *pOldBr,*pBrush;
	int Radius=GetRadius();
	switch (PegType)
	{
	case White:
		pBrush=&cbrPegWhite; break;
	case Black:
		pBrush=&cbrPegBlack; break;
	case Red:
		pBrush=&cbrPegRed; break;
	case Stricker:
		pBrush=&cbrPegStricker; break;
	}
	pOldBr=dc->SelectObject(pBrush);
	dc->Ellipse((int)xPosX-Radius,(int)xPosY-Radius,
		(int)xPosX+Radius,(int)xPosY+Radius);
	dc->SelectObject(pOldBr);
}
inline XType CPeg::GetVel()
{
	return sqrtl(GetVel2());
}
//////////////////////////////////////////////////////////////////////
// class CShot
//////////////////////////////////////////////////////////////////////
XType CShot::GotoLast()
{
	CCheckPoint *cp=GetLast();
	if (cp==NULL) return NULL;
	Restore(cp);
	return GetTime(cp);
}
void CShot::GotoTime(XType xTime)
{
	for (CCheckPoint *cp=cpTop;
		cp->Next!=cpTop && GetTime(cp->Next)<xTime;
		cp=cp->Next);
	Restore(cp);
	CBoard::TimeLapse(Pegs, 20, xTime - GetTime(cp));
}
int CShot::CountDropped(EPegTypes Type)
{
	int Result=0;
	CCheckPoint *cpLast=GetLast();
	for (int i=0; i<20; i++)
		if (Pegs[i]->PegType==Type && cpTop->nPocketNo[i]==0 && cpLast->nPocketNo[i]>0)
			Result++;
	return Result;
}
bool CShot::RedDropped()
{
	return (GetLast()->nPocketNo[18]>0 && cpTop->nPocketNo[18]==0);
}
//////////////////////////////////////////////////////////////////////
// class CBoard
//////////////////////////////////////////////////////////////////////
CBoard::CBoard(int Width,int EdgeWidth,int SLine1,int SLine2,int PocketRadius)
{
	int i;
	for (i=0; i<9; i++) Pegs[i]=new CPeg(Black);	// Black pegs:	0-8
	for (i=9; i<18; i++) Pegs[i]=new CPeg(White);	// White pegs:	9-17
	Pegs[18]=new CPeg(Red);							// Red peg:		18
	Pegs[19]=new CPeg(Stricker);					// Striker:		19
	nWidth=Width;
	nEdgeWidth=EdgeWidth;
	if (SLine1<SLine2)
	{
		nSLine1=SLine1;
		nSLine2=SLine2;
	}
	else
	{
		nSLine1=SLine2;
		nSLine2=SLine1;
	}
	nPocketRadius=PocketRadius;
	CPeg::cbrPegBlack.CreateSolidBrush(RGB(32,32,32));
	CPeg::cbrPegWhite.CreateSolidBrush(RGB(255,255,255));
	CPeg::cbrPegRed.CreateSolidBrush(RGB(255,0,0));
	CPeg::cbrPegStricker.CreateSolidBrush(RGB(255,255,0));
	xCoeffRest=0.95;
	AimArrow.x=0; AimArrow.y=0;
	Shot.SetPegs(Pegs);
}

CBoard::~CBoard()
{
	for (int i=0; i<20; i++)
		delete Pegs[i];
}

CPeg *CBoard::GetDroppedPeg(EPegTypes PegType)
{
	for (int i=0; i<20; i++)
		if (Pegs[i]->nPocketNo && Pegs[i]->PegType==PegType) return Pegs[i];
	return NULL;
}

void CBoard::DrawBoard(CDC *dc, int x, int y,DWORD DrawParam,bool bTrackLines)
{
	// Set the viewport origin to x,y to simplify matters
	CPoint Org=dc->GetViewportOrg();
	dc->SetViewportOrg(x,y);

	// Variables
	DWORD	dwLight = GetSysColor(COLOR_3DHILIGHT),
			dwDark  = GetSysColor(COLOR_3DSHADOW);
	CBrush Brush,*pOldBrush;
	CRect Rect;
	CRgn Rgn;

	if (DrawParam & DB_BOARD)
	{
		// Draw outer boundary
		Brush.CreateSolidBrush(RGB(100,100,60));
		Rect=CRect(0,0,nWidth,nWidth);
		dc->FillRect(Rect,&Brush);
		dc->Draw3dRect(Rect,dwLight,dwDark);
		Brush.DeleteObject();
		Rect=CRect(nEdgeWidth-1,nEdgeWidth-1,nWidth-nEdgeWidth+1,nWidth-nEdgeWidth+1);
		dc->Draw3dRect(Rect,dwDark,dwLight);
	}

	if (DrawParam & DB_PEGS)
	{
		// Draw inner boundary
		Brush.CreateSolidBrush(RGB(200,200,50));
		if (DrawParam & DB_BOARD) Rect.InflateRect(-1,-1);
		else Rect=CRect(nEdgeWidth,nEdgeWidth,nWidth-nEdgeWidth,nWidth-nEdgeWidth);
		dc->FillRect(Rect,&Brush);
		Brush.DeleteObject();

		// Set clipping region to inner boundary
		Rect.OffsetRect(x,y);
		Rgn.CreateRectRgnIndirect(&Rect);
		dc->SelectClipRgn(&Rgn);

		// Draw the pockets
		Brush.CreateStockObject(BLACK_BRUSH);
		pOldBrush=dc->SelectObject(&Brush);
		Rect=CRect(nEdgeWidth-nPocketRadius,nEdgeWidth-nPocketRadius,
				nEdgeWidth+nPocketRadius,nEdgeWidth+nPocketRadius);
		dc->Ellipse(&Rect);		// First pocket...
		Rect.OffsetRect(nWidth-2*nPocketRadius,0); dc->Ellipse(&Rect);	// 2nd
		Rect.OffsetRect(0,nWidth-2*nPocketRadius); dc->Ellipse(&Rect);	// 3rd
		Rect.OffsetRect(2*nPocketRadius-nWidth,0); dc->Ellipse(&Rect);	// 4th
		Brush.DeleteObject();

		// Draw the inner lines
		Brush.CreateStockObject(NULL_BRUSH);
		dc->SelectObject(Brush);
		dc->Rectangle(nSLine1,nSLine1,nWidth-nSLine1,nWidth-nSLine1);
		dc->Rectangle(nSLine2,nSLine2,nWidth-nSLine2,nWidth-nSLine2);
		dc->MoveTo(nSLine1-5,nSLine1-5);				dc->LineTo(nSLine2+10,nSLine2+10);
		dc->MoveTo(nWidth-nSLine1+5,nSLine1-5);			dc->LineTo(nWidth-nSLine2-10,nSLine2+10);
		dc->MoveTo(nSLine1-5,nWidth-nSLine1+5);			dc->LineTo(nSLine2+10,nWidth-nSLine2-10);
		dc->MoveTo(nWidth-nSLine1+5,nWidth-nSLine1+5);	dc->LineTo(nWidth-nSLine2-10,nWidth-nSLine2-10);
		Brush.DeleteObject();

		// Draw the ball
		Rect=CRect(nWidth/2,nWidth/2,nWidth/2,nWidth/2);
		Rect.InflateRect(CPeg::piGeneral.nRadius+2,CPeg::piGeneral.nRadius+2);
		dc->Ellipse(&Rect);
		Rect.InflateRect(4*CPeg::piGeneral.nRadius-3,4*CPeg::piGeneral.nRadius-3);
		dc->Ellipse(&Rect);
		Rect.InflateRect(2,2);
		dc->Ellipse(&Rect);
		Brush.CreateSolidBrush(RGB(128,100,25));
		dc->SelectObject(&Brush);
		Rect=CRect(nWidth/2,nWidth/2,nWidth/2,nWidth/2);
		Rect.InflateRect(CPeg::piGeneral.nRadius,CPeg::piGeneral.nRadius);
		dc->Ellipse(&Rect);

		// Draw the Pegs, but Stricker
		for (int i=0; i<19; i++)
			Pegs[i]->DrawPeg(dc);

		// Draw the aiming Arrow
		if (!Pegs[19]->nPocketNo && (AimArrow.x!=0 || AimArrow.y!=0))
		{
			int x=(int)Pegs[19]->xPosX, y=(int)Pegs[19]->xPosY;
			
			CPen Pen(PS_SOLID,0,RGB(220,220,220)),*pOldPen;
			pOldPen=dc->SelectObject(&Pen);

			int DiffX=AimArrow.AimX-AimArrow.x,DiffY=AimArrow.AimY-AimArrow.y;
			dc->MoveTo(x+AimArrow.x1+DiffX,y+AimArrow.y1+DiffY);
			dc->LineTo(x+AimArrow.AimX,y+AimArrow.AimY);
			dc->MoveTo(x+AimArrow.x2+DiffX,y+AimArrow.y2+DiffY);
			dc->LineTo(x+AimArrow.AimX,y+AimArrow.AimY);

			dc->SelectObject(pOldPen);
			Pen.DeleteObject();
			Pen.CreatePen(PS_SOLID,0,RGB(220,220,255));
			pOldPen=dc->SelectObject(&Pen);

			dc->MoveTo(x+AimArrow.x1,y+AimArrow.y1);
			dc->LineTo(x+AimArrow.x,y+AimArrow.y);
			dc->MoveTo(x+AimArrow.x2,y+AimArrow.y2);
			dc->LineTo(x+AimArrow.x,y+AimArrow.y);
			dc->MoveTo(x,y);
			dc->LineTo(x+AimArrow.x,y+AimArrow.y);

			dc->SelectObject(pOldPen);
		}

		//Draw the Stricker
		Pegs[19]->DrawPeg(dc);

		//Draw tracking lines
		if (bTrackLines)
		{
			int nCount=Shot.GetCount();
			XType x,y;
			int nPocketNo;
			CPen Pen,*pOldPen;
			Pen.CreatePen(PS_SOLID,0,RGB(32,32,32));
			for (int i=0; i<20; i++)
			{
				if (i==9) 
				{
					Pen.DeleteObject();
					Pen.CreatePen(PS_SOLID,0,RGB(255,255,255));
				}
				else if (i==18) 
				{
					Pen.DeleteObject();
					Pen.CreatePen(PS_SOLID,0,RGB(255,0,0));
				}
				else if (i==19) 
				{
					Pen.DeleteObject();
					Pen.CreatePen(PS_SOLID,0,RGB(255,255,100));
				}
				Shot.GetInfo(0,i,&x,&y,&nPocketNo);
				if (nPocketNo) continue;
				pOldPen=dc->SelectObject(&Pen);
				dc->MoveTo((int)x,(int)y);
				for (int j=1; j<nCount; j++)
				{
					Shot.GetInfo(j,i,&x,&y,&nPocketNo);
					dc->LineTo((int)x,(int)y);
					if (nPocketNo) break;
				}
				dc->SelectObject(pOldPen);
			}
		}

		dc->SelectObject(pOldBrush);

		// Reset clipping region
		dc->SelectClipRgn(NULL);
	}

	// Reset viewport origin
	dc->SetViewportOrg(Org);
}

void CBoard::ArrangeBall(XType Degrees,bool Additive)
{
	static double Deg;
	int i;
	double Freedom=1.05,d=M_PI*Degrees/180;
	if (Additive) Deg+=d;
	else Deg=d;
	for (i=0; i<6; i++)
	{
		int j= (i%2==0) ? i/2+9 : i/2;
		d=Deg+M_PI*i/3;
		Pegs[j]->xPosX = nWidth/2 + cosl(d)*2*CPeg::piGeneral.nRadius*Freedom;
		Pegs[j]->xPosY = nWidth/2 + sinl(d)*2*CPeg::piGeneral.nRadius*Freedom;
		Pegs[j]->Rest();
		Pegs[j]->nPocketNo=0;
	}
	for (i=3; i<9; i++)
	{
		d=Deg+M_PI*i/3+M_PI/6;
		Pegs[i]->xPosX = nWidth/2 + cosl(d)*2*M_SQRT_3*CPeg::piGeneral.nRadius*Freedom;
		Pegs[i]->xPosY = nWidth/2 + sinl(d)*2*M_SQRT_3*CPeg::piGeneral.nRadius*Freedom;
		Pegs[i]->Rest();
		Pegs[i]->nPocketNo=0;
	}
	for (i=12; i<18; i++)
	{
		d=Deg+M_PI*i/3;
		Pegs[i]->xPosX = nWidth/2 + cosl(d)*4*CPeg::piGeneral.nRadius*Freedom;
		Pegs[i]->xPosY = nWidth/2 + sinl(d)*4*CPeg::piGeneral.nRadius*Freedom;
		Pegs[i]->Rest();
		Pegs[i]->nPocketNo=0;
	}
	Pegs[18]->xPosX=nWidth/2; Pegs[18]->xPosY=nWidth/2;
	Pegs[18]->nPocketNo=0;
}

void CBoard::CollusionTest(CBoard::CCollusion *pInfo)
{
	int i,j;
	static XType xPosX[20],xPosY[20];
	ZeroMemory(pInfo, sizeof(CCollusion));
	for (i=0; i<20 && pInfo->Peg1==NULL; i++)
	{
		if (Pegs[i]->nPocketNo) continue;
		XType xi=Pegs[i]->xPosX, yi=Pegs[i]->xPosY;
		if (xPosX[i]==xi && xPosY[i]==yi) continue;
		
		for (j=0; j<4; j++)
		{
			XType dx,dy;
			if (j%2) dx=nEdgeWidth-xi;
			else dx=nWidth-nEdgeWidth-xi;
			if (j/2) dy=nEdgeWidth-yi;
			else dy=nWidth-nEdgeWidth-yi;
			if (dx*dx+dy*dy<nPocketRadius*nPocketRadius)
			{
				pInfo->Peg1=Pegs[i];
				pInfo->Dropped=j+1;
				break;
			}
		}
		if (pInfo->Peg1!=NULL) break;

		int ri=Pegs[i]->GetRadius();
		if (xi-ri<nEdgeWidth ||
			yi-ri<nEdgeWidth ||
			xi+ri>nWidth-nEdgeWidth ||
			yi+ri>nWidth-nEdgeWidth)
		{
			pInfo->Peg1=Pegs[i];
			break;
		}
		
		for (j=0; j<20; j++)
		{
			if (j==i) continue;
			if (Pegs[j]->nPocketNo) continue;
			if (j<i && (Pegs[j]->xPosX!=xPosX[j] || Pegs[j]->xPosY!=xPosY[j])) continue;
			XType dx=xi-Pegs[j]->xPosX, dy=yi-Pegs[j]->xPosY;
			int rTotal=ri+Pegs[j]->GetRadius();
			XType r2=rTotal*rTotal-1e-8;
			if (dx*dx+dy*dy<r2)
			{
				pInfo->Peg1=Pegs[i];
				pInfo->Peg2=Pegs[j];
				break;
			}
		}
	}
	if (pInfo->Peg1==NULL)
		for (int i=0; i<20; i++)
		{
			xPosX[i]=Pegs[i]->xPosX;
			xPosY[i]=Pegs[i]->xPosY;
		}
}

void CBoard::TimeLapse(CPeg **Pegs, int NumPegs, XType Time)
{
	for (int i=0; i<NumPegs; i++)
	{
		XType Accel,Vel;
		Vel=Pegs[i]->GetVel();
		if (Vel>0)
		{
			Accel=Pegs[i]->GetAccel();
			// -Vel/Accel is the time required for coming to full stop
			if (-Vel/Accel>Time)	// ie. it will not stop in given time
			{
				XType AccelX,AccelY;
				AccelX=Pegs[i]->xVelX/Vel*Accel;
				AccelY=Pegs[i]->xVelY/Vel*Accel;
				Pegs[i]->xPosX+=Pegs[i]->xVelX*Time+AccelX*Time*Time/2;
				Pegs[i]->xPosY+=Pegs[i]->xVelY*Time+AccelY*Time*Time/2;
				Pegs[i]->xVelX+=AccelX*Time;
				Pegs[i]->xVelY+=AccelY*Time;
			}
			else					// ie. it will stop withinin given time
			{
				XType Dist=-Vel*Vel/2/Accel;
				Pegs[i]->xPosX+=Pegs[i]->xVelX/Vel*Dist;
				Pegs[i]->xPosY+=Pegs[i]->xVelY/Vel*Dist;
				Pegs[i]->xVelX=Pegs[i]->xVelY=0;
			}
		}
		if (Pegs[i]->xSpin>0)
		{
			// How does the spin decrease?
		}
	}
}

void CBoard::MoveStricker(XType x,XType y,bool Additive)
{
	CPeg* Stricker=Pegs[19];
	int Radius=Stricker->GetRadius();
	int MinY=nWidth-nSLine1-Radius-1;
	int MaxY=nWidth-nSLine2+Radius;
	int MinX=nWidth-MaxY+int(M_SQRT_2*Radius)+1;
	int MaxX=MaxY-int(M_SQRT_2*Radius);
	if (Additive)
	{
		x+=Stricker->xPosX;
		y+=Stricker->xPosY;
	}
	Stricker->xPosX=x; Stricker->xPosY=y;
	if (x > MaxX) Stricker->xPosX=x=MaxX;
	else if (x < MinX) Stricker->xPosX=x=MinX;

	int MinY_2;
	MinY_2=int(x+Radius*M_SQRT_2);
	if (MinY_2>MinY) MinY=MinY_2;
	MinY_2=int(nWidth-x+Radius*M_SQRT_2)+1;
	if (MinY_2>MinY) MinY=MinY_2;
	if (y > MaxY) Stricker->xPosY=MaxY;
	else if (y < MinY) Stricker->xPosY=MinY;
}

bool CBoard::IsInMotion()
{
	for (int i=0; i<20; i++)
	{
		if (Pegs[i]->nPocketNo) continue;
		if (Pegs[i]->IsInMotion()) return true;
	}
	return false;
}

XType CBoard::GetMaxVel()
{
	XType Result=0;
	for (int i=0; i<20; i++)
	{
		if (Pegs[i]->nPocketNo) continue;
		XType Vel2 = Pegs[i]->GetVel2();
		if (Vel2>Result) Result=Vel2;
	}
	return sqrtl(Result);
}

XType CBoard::GenerateShot(int Max)
{
	CCollusion ColInfo;
	CollusionTest(&ColInfo);
	if (ColInfo.Peg1!=NULL) return -1;
	XType Time=0,MaxVel;
	Shot.ClearAll();
	Shot.AddCheckPoint(0);
	while ((MaxVel=GetMaxVel())>0)
	{
		XType Period=1.0L/MaxVel;
		while (IsInMotion())
		{
			TimeLapse(Pegs,20,Period); Time+=Period;
			CollusionTest(&ColInfo);
			if (ColInfo.Peg1!=NULL) 
			{
				Time-=Period;
				XType OldTime=Shot.GotoLast();
				TimeLapse(Pegs,20,Time-OldTime);
				if (Period>1e-7) Period/=2;
				else break;
			}
		}
		if (Collide(&ColInfo)) Shot.AddCheckPoint(Time);
		if (Max && Shot.GetCount()>Max) break;
	}
	Shot.AddCheckPoint(Time);
	Shot.GotoTime(0);
	return Time;
}

bool CBoard::Collide(CBoard::CCollusion *pInfo)
{
	CPeg* Peg1=pInfo->Peg1;
	CPeg* Peg2=pInfo->Peg2;
	if (Peg1==NULL) return false;
	if (pInfo->Dropped)
	{
		Peg1->nPocketNo=pInfo->Dropped;
	}
	else if (Peg2==NULL)	// Collusion with side
	{
		XType x=Peg1->xPosX,y=Peg1->xPosY;
		if (y<x)	// Right or top
		{
			if (y>nWidth-x) Peg1->xVelX=-Peg1->xVelX*xCoeffRest;
			else Peg1->xVelY=-Peg1->xVelY*xCoeffRest;
		}
		else		// Left or bottom
		{
			if (y<nWidth-x) Peg1->xVelX=-Peg1->xVelX*xCoeffRest;
			else Peg1->xVelY=-Peg1->xVelY*xCoeffRest;
		}
	}
	else	// Two pegs collide
	{
		XType Theta=atan2l(Peg2->xPosY-Peg1->xPosY,Peg2->xPosX-Peg1->xPosX);
		XType s=sin(Theta),c=cos(Theta);
		XType m1=Peg1->GetMass(),m2=Peg2->GetMass();
		XType ux1,uy1,ux2,uy2;	// Initial velocities
		XType vx1,vy1,vx2,vy2;	// Final velocities
		// Rotate the axes
		ux1 = Peg1->xVelX*c+Peg1->xVelY*s;
		uy1 =-Peg1->xVelX*s+Peg1->xVelY*c;
		ux2 = Peg2->xVelX*c+Peg2->xVelY*s;
		uy2 =-Peg2->xVelX*s+Peg2->xVelY*c;
		// Operate on this axes
		vx1=(ux1-ux2)*(m1-m2*xCoeffRest)/(m1+m2)+ux2;
		vy1=uy1;
		vx2=(ux1-ux2)*m1*(1+xCoeffRest)/(m1+m2)+ux2;
		vy2=uy2;
		// Rotate back to original
		Peg1->xVelX = vx1*c-vy1*s;
		Peg1->xVelY = vx1*s+vy1*c;
		Peg2->xVelX = vx2*c-vy2*s;
		Peg2->xVelY = vx2*s+vy2*c;
	}
	return true;
}

void CBoard::PutStricker()
{
	Pegs[19]->nPocketNo=0;
	MoveStricker(nWidth/2+rand()%11-5,nWidth-(nSLine1+nSLine2)/2,false);
}

void CBoard::Rotate(XType CosTheta,XType SinTheta)
{
	for (int i=0; i<20; i++)
	{
		if (Pegs[i]->nPocketNo) continue;
		XType a,x=Pegs[i]->xPosX-nWidth/2,y=Pegs[i]->xPosY-nWidth/2;
		a=				CosTheta*x - SinTheta*y + nWidth/2;
		Pegs[i]->xPosY= SinTheta*x + CosTheta*y + nWidth/2;
		Pegs[i]->xPosX=a;
		a=				CosTheta*Pegs[i]->xVelX-SinTheta*Pegs[i]->xVelY;
		Pegs[i]->xVelY= SinTheta*Pegs[i]->xVelX+CosTheta*Pegs[i]->xVelY;
		Pegs[i]->xVelX=a;
	}
}

void CBoard::MoveSpotted(CPeg *Peg, XType dx, XType dy)
{
	XType Rad2,MaxRad=4*CPeg::piGeneral.nRadius+1;
	XType x=Peg->xPosX - nWidth/2, y=Peg->xPosY - nWidth/2;
	x+=dx; y+=dy;
	Rad2=x*x+y*y;
	if (Rad2>MaxRad*MaxRad)
	{
		XType Radius=sqrtl(Rad2);
		x=x*MaxRad/Radius;
		y=y*MaxRad/Radius;
	}
	Peg->xPosX = x + nWidth/2;
	Peg->xPosY = y + nWidth/2;
}

void CBoard::SpotPeg(CPeg **pPeg)
{
	CPeg* Peg=*pPeg;
	bool Done=false;
	Peg->Rest();
	Peg->nPocketNo=0;
	for (int Dist=0; !Done; Dist++)
	{
		// Incr should be 1/(2*M_PI*Dist) to cover each pixel
		XType Incr=(Dist==0 ? 1024 : 0.16/Dist);
		for (XType Angle=0; !Done && Angle<2*M_PI; Angle+=Incr)
		{
			Peg->xPosX=cosl(Angle)*Dist+nWidth/2;
			Peg->xPosY=sinl(Angle)*Dist+nWidth/2;
			for (int i=0; !Done && i<20; i++)
			{
				if (Pegs[i]==Peg) continue;
				if (Pegs[i]->nPocketNo>0) continue;
				int nTotalRad2=Peg->GetRadius()+Pegs[i]->GetRadius();
				nTotalRad2=nTotalRad2*nTotalRad2;
				XType dx=Pegs[i]->xPosX - Peg->xPosX;
				XType dy=Pegs[i]->xPosY - Peg->xPosY;
				XType Rad2=dx*dx+dy*dy;;
				if (Rad2<nTotalRad2) break;
			}
			if (i==20) Done=true;
		}
	}
}
