// Board.h: interface for the CBoard class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BOARD_H__53FABBB1_658E_11D5_8547_DD80BF6AEF38__INCLUDED_)
#define AFX_BOARD_H__53FABBB1_658E_11D5_8547_DD80BF6AEF38__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//////////////////////////////////////////////////////////////////////
// Start of my code
//////////////////////////////////////////////////////////////////////
typedef long double XType;

enum EPegTypes {Black=0,White=1,Red,Stricker};

//////////////////////////////////////////////////////////////////////
// class CPeg
//////////////////////////////////////////////////////////////////////
class CPeg
{
public:
	static struct PegInfo {
		int nRadius;	// Radius
		XType xMass;	// Masses
		XType xAccel;	// Decceleration due to friction, must be -ve
	} piGeneral,piStricker;
	static CBrush cbrPegWhite,cbrPegBlack,cbrPegRed,cbrPegStricker;	// To be initialized in CBoard
	EPegTypes PegType;
	int nPocketNo;		// Pocket no, 0 if OnBoard
	XType xPosX,xPosY;
	XType xVelX,xVelY;
	XType xSpin;

	CPeg(EPegTypes Type);
	void DrawPeg(CDC *dc);
	int GetRadius() {return (PegType==Stricker) ? piStricker.nRadius : piGeneral.nRadius;}
	XType GetMass() {return (PegType==Stricker) ? piStricker.xMass : piGeneral.xMass;}
	XType GetAccel() {return (PegType==Stricker) ? piStricker.xAccel : piGeneral.xAccel;}
	XType GetVel2() {return xVelX*xVelX+xVelY*xVelY;}
	XType GetVel();
	bool IsInMotion() {return (xVelX!=0 || xVelY!=0);}
	void Rest() {xVelX=xVelY=xSpin=0;}
};

//////////////////////////////////////////////////////////////////////
// class CHistory
//////////////////////////////////////////////////////////////////////
template <class T> class CHistory
{
protected:
	class CCheckPoint {
	public:
		int nPocketNo[20];
		XType xPosX[20],xPosY[20];
		XType xVelX[20],xVelY[20];
		XType xSpin[20];
		CCheckPoint *Prev;
		CCheckPoint *Next;
		T *pObject;
		CCheckPoint() {pObject=NULL;}
		~CCheckPoint() {delete pObject;}
	};
	CPeg **Pegs;
	CCheckPoint *AddElement();
	CCheckPoint *GetLast() {return nCount==0 ? NULL : cpTop->Prev;}
	void Restore(CCheckPoint *cp);
	CCheckPoint *cpTop;
	CCheckPoint *GetAt(int Num);
	int nCount;
public:
	CHistory() {nCount=0;}
	virtual ~CHistory() {ClearAll();}
	void SetPegs(CPeg **ThePegs) {Pegs=ThePegs;}
	CCheckPoint *AddCheckPoint();
	void ClearAll();
	void RemoveLast();
	void operator =(CHistory &Ref);
	void GetInfo(int FrameNo, int PegNo, XType *xPosX, XType *xPosY, int *nPocketNo);
	int GetCount() {return nCount;}
	void GotoLast() {if (nCount!=0) Restore(GetLast());}
};
//////////////////////////////////////////////////////////////////////
// class CShot
//////////////////////////////////////////////////////////////////////
class CShot : public CHistory<XType>
{
private:
	XType GetTime(CHistory<XType>::CCheckPoint *cp) {return *cp->pObject;}
	void SetTime(CHistory<XType>::CCheckPoint *cp,XType Time) {*cp->pObject=Time;}
public:
	void AddCheckPoint(XType xTime) {SetTime(CHistory<XType>::AddCheckPoint(),xTime);}
	void GotoTime(XType xTime);
	XType GotoLast();	// Returns time
	// Returns no. of pegs dropped *in this shot*
	int CountDropped(EPegTypes Type);
	// Returns true if Red pocketed *in this shot*
	bool RedDropped();
};

//////////////////////////////////////////////////////////////////////
// class CBoard
//////////////////////////////////////////////////////////////////////
#define DB_BOARD	1
#define DB_PEGS		2
#define DB_ALL		(DB_BOARD | DB_PEGS)
class CBoard  
{
protected:
	XType xCoeffRest;	// Coefficient of restitution
	int nWidth,nEdgeWidth,nSLine1,nSLine2,nPocketRadius;
	struct CCollusion
	{
		CPeg *Peg1,*Peg2;
		int Dropped;	// Means Peg1 is in pocket, Peg2=NULL
	};
	bool Collide(CCollusion *pInfo);
	void CollusionTest(CCollusion *pInfo);
	bool IsInMotion();	// Returns true if something has non-0 vel
	XType GetMaxVel();	// Returns the maximum velocity
public:
	static void TimeLapse(CPeg **Pegs,int NumPegs,XType Time);

	struct CAimArrow {
		int x,y;		// Tip of the arrow (starting from 0,0)
		int x1,y1,x2,y2;
		int AimX,AimY;
	} AimArrow;
	CShot Shot;
	CPeg* Pegs[20];

	CBoard(int Width,int EdgeWidth,int SLine1,int SLine2,int PocketRadius);
	~CBoard();
	CPeg** GetPegs() {return Pegs;}
	CPeg* GetStricker() {return Pegs[19];}
	CPeg* GetRed() {return Pegs[18];}
	CPeg *GetDroppedPeg(EPegTypes PegType);	// Returns a Peg with nPocketNo>0
	void DrawBoard(CDC *dc, int x, int y,DWORD DrawParam,bool bTrackLines=false);	// DB_??? as DrawParam
	void ArrangeBall(XType Degrees,bool Additive);
	void MoveStricker(XType x,XType y,bool Additive);
	// Takes Maximum # collusions as (optional) input,
	// Returns -1 if collusion detected in beginning.
	XType GenerateShot(int Max=0);
	void PutStricker();
	void Rotate(XType CosTheta,XType SinTheta);
	void MoveSpotted(CPeg *Peg,XType dx,XType dy);
	// Internally changes *pPeg (and puts the topmost matching Peg there)
	void SpotPeg(CPeg **pPeg);
};

//////////////////////////////////////////////////////////////////////
// class CHistory template code
//////////////////////////////////////////////////////////////////////
template <class T> CHistory<T>::CCheckPoint* CHistory<T>::AddElement()
{
	CCheckPoint *cpNew=new CCheckPoint;
	if (nCount==0)
	{
		cpTop=cpNew;
		cpTop->Prev=cpTop->Next=cpTop;
	}
	else
	{
		cpTop->Prev->Next=cpNew; cpNew->Prev=cpTop->Prev;
		cpTop->Prev=cpNew; cpNew->Next=cpTop;
	}
	nCount++;
	return cpNew;
}

template <class T> CHistory<T>::CCheckPoint* CHistory<T>::AddCheckPoint()
{
	CCheckPoint *cpNew=AddElement();
	for (int i=0; i<20; i++)
	{
		cpNew->xPosX[i]=Pegs[i]->xPosX;
		cpNew->xPosY[i]=Pegs[i]->xPosY;
		cpNew->xVelX[i]=Pegs[i]->xVelX;
		cpNew->xVelY[i]=Pegs[i]->xVelY;
		cpNew->xSpin[i]=Pegs[i]->xSpin;
		cpNew->nPocketNo[i]=Pegs[i]->nPocketNo;
	}
	cpNew->pObject=new T;
	return cpNew;
}

template <class T> void CHistory<T>::ClearAll()
{
	if (nCount==0) return;
	while (cpTop->Next!=cpTop)
	{
		CCheckPoint *cpNext=cpTop->Next;
		cpTop->Next=cpNext->Next;
		delete cpNext;
	}
	delete cpTop;
	nCount=0;
}

template <class T> void CHistory<T>::Restore(CHistory<T>::CCheckPoint *cp)
{
	for (int i=0; i<20; i++)
	{
		Pegs[i]->xPosX=cp->xPosX[i];
		Pegs[i]->xPosY=cp->xPosY[i];
		Pegs[i]->xVelX=cp->xVelX[i];
		Pegs[i]->xVelY=cp->xVelY[i];
		Pegs[i]->xSpin=cp->xSpin[i];
		Pegs[i]->nPocketNo=cp->nPocketNo[i];
	}
}

template <class T> void CHistory<T>::RemoveLast()
{
	if (nCount==0) return;
	CCheckPoint *Prev=cpTop->Prev;
	cpTop->Prev=Prev->Prev;
	cpTop->Prev->Next=cpTop;
	delete Prev;
	nCount--;
}

template <class T> void CHistory<T>::operator =(CHistory<T> &Ref)
{
	ClearAll();
	CCheckPoint *cpRef=Ref.cpTop;
	for (int i=0; i<Ref.nCount; i++)
	{
		CCheckPoint *cpNew=AddElement(),*prev,*next;
		prev=cpNew->Prev; next=cpNew->Next;
		*cpNew=*cpRef;
		cpNew->Prev=prev; cpNew->Next=next;
		cpNew->pObject=new T;	// AddElement does not create the object
		*(cpNew->pObject)=*(cpRef->pObject);
		cpRef=cpRef->Next;
	}
	Pegs=Ref.Pegs;
}

template <class T> CHistory<T>::CCheckPoint* CHistory<T>::GetAt(int Num)
{
	if (nCount==0) return NULL;
	CCheckPoint *RetVal=cpTop;
	for (int i=0; i<Num; i++)
	{
		RetVal=RetVal->Next;
		if (RetVal==cpTop) 
		{
			RetVal=NULL;
			break;
		}
	}
	return RetVal;
}

template <class T> void CHistory<T>::GetInfo(int FrameNo, int PegNo, XType *xPosX, XType *xPosY, int *nPocketNo)
{
	CCheckPoint *cp=GetAt(FrameNo);
	*xPosX=cp->xPosX[PegNo];
	*xPosY=cp->xPosY[PegNo];
	*nPocketNo=cp->nPocketNo[PegNo];
}
//////////////////////////////////////////////////////////////////////
// End of my code
//////////////////////////////////////////////////////////////////////

#endif // !defined(AFX_BOARD_H__53FABBB1_658E_11D5_8547_DD80BF6AEF38__INCLUDED_)
