// Game.cpp: implementation of the CSound class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Carrom.h"
#include "Game.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include <Math.h>
//////////////////////////////////////////////////////////////////////
// CRule and inherited classes
//////////////////////////////////////////////////////////////////////
CRule::CRule(CBoard *Board)
{
	TheBoard=Board;
	nCurPlayer=0;
	nSpotted=0;
	for (int i=0; i<4; i++)
	{
		Players[i].bIsHuman=Players[i].bExists=true;
		Players[i].nColor=(i+1)%2;
		Players[i].nAimRad=55;
	}
}
void CRule::OnGameStart()
{
	nCurPlayer=0;
	nSpotted=0;
}

// CRule1 ////////////////////////////////////////////////////////////
void CRule1::OnGameStart()
{
	CRule::OnGameStart();
	TheBoard->ArrangeBall(0,false);
	GameStage=gsArranging;
}
void CRule1::SpotPeg(CPeg *Peg)
{
	if (Peg==NULL)
	{
		Peg=TheBoard->GetDroppedPeg(Players[nCurPlayer].GetPegType());
		if (Peg==NULL)
		{
			Peg=TheBoard->GetRed();
			if (Peg->nPocketNo==0 || 
				nRedTakenBy!=Players[nCurPlayer].GetPegType())
				return;
		}
	}
	TheBoard->SpotPeg(&Peg);
	SpottedPegs[nSpotted++]=Peg;
}
bool CRule1::OnEndShot(CShot &Shot)
{
	bool NextPlayer=false;
	int CountDropped=Shot.CountDropped(Players[nCurPlayer].GetPegType());
	bool bRedDropped=Shot.RedDropped();
	if (bRedDropped) nRedTakenBy=Players[nCurPlayer].GetPegType();
	nSpotted=0;		// Reset the no. of spotted pegs
	if (CountDropped==0 && !bRedDropped) NextPlayer=true;
	if (TheBoard->GetStricker()->nPocketNo!=0)	// A fine
	{
		CPeg* Dropped=NULL;
		// Spot Red if fined with Red
		if (bRedDropped) Dropped=TheBoard->GetRed();
		SpotPeg(Dropped);
		if (CountDropped) SpotPeg();	// Double fine
	}
	if (NextPlayer) 
	{
		TheBoard->Rotate(0,1);
		nCurPlayer=(nCurPlayer+1)%4;
	}
	return true;
}

//////////////////////////////////////////////////////////////////////
// CGame
//////////////////////////////////////////////////////////////////////
CGame::CGame()
{
	TheBoard=new CBoard(400,20,50,60,20);
	bTrackLines=false;
	fReplayRate=1;
	History.SetPegs(TheBoard->GetPegs());
	Rule=new CRule1(TheBoard);
	Rule->GameStage=gsNone;
	Invalidate();
}
CGame::~CGame()
{
	delete TheBoard;
	delete Rule;
}
void CGame::ApplyVel()
{
	TheBoard->GetStricker()->xVelX=xVelocity*cosl(xAimAngle);
	TheBoard->GetStricker()->xVelY=xVelocity*sinl(xAimAngle);
	Invalidate();
}
void CGame::AimAt(XType Degrees,bool Additive)
{
	int nRad=Rule->Players[Rule->nCurPlayer].nAimRad+TheBoard->GetStricker()->GetRadius();
	if (Additive) xAimAngle+=Degrees*M_PI/180;
	else xAimAngle=Degrees*M_PI/180;
	TheBoard->AimArrow.x = int(nRad*cos(xAimAngle));
	TheBoard->AimArrow.y = int(nRad*sin(xAimAngle));
	TheBoard->AimArrow.x1= int((nRad-5)*cos(xAimAngle-0.1));
	TheBoard->AimArrow.y1= int((nRad-5)*sin(xAimAngle-0.1));
	TheBoard->AimArrow.x2= int((nRad-5)*cos(xAimAngle+0.1));
	TheBoard->AimArrow.y2= int((nRad-5)*sin(xAimAngle+0.1));
	TheBoard->AimArrow.AimX = int((nRad-xHitPosition)*cos(xAimAngle));
	TheBoard->AimArrow.AimY = int((nRad-xHitPosition)*sin(xAimAngle));
	ApplyVel();
}
void CGame::MoveSpotted(int Num,XType dx,XType dy)
{
	if (Num>=Rule->nSpotted) return;
	TheBoard->MoveSpotted(Rule->SpottedPegs[Num],dx,dy);
	Invalidate();
}
void CGame::AimAtMoveBy(XType x,XType y)
{
	XType Degrees;
	Degrees=cos(atan2(-x,y)-xAimAngle)*hypot(x,y);
	AimAt(Degrees,true);
}
void CGame::ChangeVel(XType y,bool Additive)
{
	if (Additive) xVelocity-=y;
	else xVelocity=y;
	if (xVelocity>1000) xVelocity=1000;
	else if (xVelocity<50) xVelocity=50;
	ApplyVel();
}
void CGame::HitPositionChange(XType y,bool Additive)
{
	static XType Positions[5];
	static float Times[5];
	float Current;
	int i;
	if (Additive) xHitPosition+=y;
	else xHitPosition=y;

	Current=Timer.Current();
	if (xHitPosition<0)
	{
		if (Current-Times[0]<=.1)
		{
			for (i=0; i<4; i++)
				if (Current-Times[i+1]>.1) break;
			ChangeVel((Positions[i]-xHitPosition)/(Current-Times[i]),false);
			NextStage();
		}
		else xHitPosition=0;
	}
	else 
	{
		int nRad=Rule->Players[Rule->nCurPlayer].nAimRad;
		if (xHitPosition>nRad) xHitPosition=nRad;
		nRad+=TheBoard->GetStricker()->GetRadius();
		TheBoard->AimArrow.AimX = int((nRad-xHitPosition)*cos(xAimAngle));
		TheBoard->AimArrow.AimY = int((nRad-xHitPosition)*sin(xAimAngle));
	}

	for (i=4; i>=1; i--)
	{
		Positions[i]=Positions[i-1];
		Times[i]=Times[i-1];
	}
	Positions[0]=xHitPosition;
	Times[0]=Current;
	Invalidate();
}
void CGame::DrawBoard(CDC *dc,int x,int y,bool DrawAll)
{
	static bool bAimDrawn;
	bool Draw=true;
	DWORD DrawParam;
	if (DrawAll) {DrawParam=DB_ALL; bAimDrawn=false;}
	else DrawParam=DB_PEGS;
	if (Rule->GameStage==gsArranging || Rule->GameStage==gsAiming) 
	{
		static DWORD LastChangeTime;
		if (bNeedsRedraw)
		{
			LastChangeTime=GetTickCount();
			bAimDrawn=false;
			bNeedsRedraw=false;
		}
		int Num=GetTickCount()-LastChangeTime<100?5:0;
		if (!bAimDrawn)
			TheBoard->DrawBoard(dc,x,y,DrawParam,bTrackLines && TheBoard->GenerateShot(Num)!=-1);
		if (Num==0) bAimDrawn=true;
		Draw=false;
	}
	else if (gsPlayingShot())	// Playing shot
	{
		float Time=Timer.GetPeriod();
		shReplay.GotoTime(Time);
		if (Time>m_xShotTime) NextStage();
	}
	else if (Rule->GameStage==gsNone && !DrawAll) Draw=false;
	if (Draw) TheBoard->DrawBoard(dc,x,y,DrawParam);
}
void CGame::NextStage()
{
	switch (Rule->GameStage)
	{
	case gsNone:
		MessageBox(NULL,"NextStage called when GameStage == gsNone.","Error",MB_OK|MB_ICONWARNING);
		break;
	case gsArranging:
	case gsAiming:
	{
		XType ShotTime=TheBoard->GenerateShot();
		if (ShotTime!=-1)
		{
			m_xShotTime=ShotTime;
			TheBoard->AimArrow.x=TheBoard->AimArrow.y=0;
			shReplay=TheBoard->Shot;
			Timer.SetRate(1);
			Timer.Start();
			Rule->GameStage=gsPlaying;
		}
		else
		{
			ShowCursor(TRUE);
			AfxGetApp()->m_pMainWnd->MessageBox("Could not initiate shot.\n"
				"Please make sure there are no ovelaps.","Error",
				MB_OK | MB_ICONINFORMATION);
			HitPositionChange(0,false);
			ShowCursor(FALSE);
			Invalidate();
		}
		break;
	}

	case gsPlaying:
	{
		bool Tossing=!Rule->OnEndShot(shReplay);
		TheBoard->PutStricker();
		AimAt(-rand()%61-60,false);
		if (Tossing) {Rule->GameStage=gsAiming; break;}
		History.AddCheckPoint();
		HitPositionChange(0,false);
	}
	case gsReplaying:
		History.GotoLast();
		AimAt(0,true);
		Rule->GameStage=gsAiming;
		break;
	}
}
void CGame::Replay()
{
	if (!gsPlayingShot() && GameStage!=gsAiming) return;
	if (GameStage==gsAiming)
	{
		// To note the position of stricker again
		History.RemoveLast();
		History.AddCheckPoint();
	}
	TheBoard->AimArrow.x=TheBoard->AimArrow.y=0;
	Timer.SetRate(fReplayRate);
	Timer.Start();
	if (GameStage!=gsPlaying) GameStage=gsReplaying;
}
void CGame::ReplayRatePlus() 
{
	if (fReplayRate<0.9999) fReplayRate*=2;
	if (gsPlayingShot()) Timer.SetRate(fReplayRate);
}
void CGame::ReplayRateMinus() 
{
	if (fReplayRate>0.1251) fReplayRate/=2;
	if (gsPlayingShot()) Timer.SetRate(fReplayRate);
}

void CGame::NewGame(CRule *NewRule)
{
	// Starting a game.
	delete Rule; Rule=NewRule;
	Rule->OnGameStart();
	TheBoard->PutStricker();
	AimAt(-rand()%61-60,false);
	ChangeVel(500+rand()%11-5,false);
	HitPositionChange(0,false);
	History.ClearAll();
	History.AddCheckPoint();
}
