// Game.h: interface for the CSound class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GAME_H__70431C42_7436_11D5_8548_444553540000__INCLUDED_)
#define AFX_GAME_H__70431C42_7436_11D5_8548_444553540000__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Board.h"
#include "Utils.h"
#include "SoundStream.h"

//////////////////////////////////////////////////////////////////////
// EGameStage denotes current stage
//////////////////////////////////////////////////////////////////////
enum EGameStage {gsNone, gsArranging, gsAiming, gsPlaying, gsReplaying};

//////////////////////////////////////////////////////////////////////
// CRule
//////////////////////////////////////////////////////////////////////
class CRule
{
public:
	CBoard *TheBoard;
	struct CPlayer {
		bool bExists;
		bool bIsHuman;
		int nAimRad;
		int nColor;		// 0 - Black, 1 - White
		EPegTypes GetPegType() {return nColor==0?Black:White;}
	} Players[4];
	int nCurPlayer;
	int nSpotted;	// Maximum 2 pegs can be spotted, and arranged arbitrarily
	CPeg* SpottedPegs[2];
	EGameStage GameStage;

	CRule(CBoard *Board);
	virtual ~CRule() {}
	virtual void OnGameStart();
	// Returns false if tossing, true otherwise
	virtual bool OnEndShot(CShot &Shot)=0;
};
class CRule1 : public virtual CRule
{
private:
	EPegTypes nRedTakenBy;	// Meaningful only when Red is in a pocket
	void SpotPeg(CPeg *Peg=NULL);
public:
	CRule1(CBoard *TheBoard) : CRule(TheBoard) {}
	void OnGameStart();
	bool OnEndShot(CShot &Shot);
};
//////////////////////////////////////////////////////////////////////
// CGame
//////////////////////////////////////////////////////////////////////
#ifdef _SomethingIWontDefineEver_
#define _CLW_HIDE {
#define _CLW_UNHIDE }
#else
#define _CLW_HIDE
#define _CLW_UNHIDE
#endif
class CGame
{
private:
	bool bNeedsRedraw;	// Use the Invalidate() method, don't use directly
	XType xAimAngle;
	XType xVelocity;	// Velocity for current hit
	float fReplayRate;
	XType xHitPosition;
	CHistory<char> History;
	CShot shReplay;
	CSoundStream SoundStream;
	CRule* Rule;

	void ApplyVel();	// Call ApplyVel whenever xAimAngle or xVelocity change
	void AimAt(XType Degrees,bool Additive);

public:
	void NewGame(CRule *Rule);
	bool bTrackLines;
	XType m_xShotTime;
	CTimer Timer;
	CBoard* TheBoard;

	CGame();
	~CGame();
	void NextStage();
	void MoveSpotted(int Num,XType dx,XType dy);
	void AimAtMoveBy(XType x,XType y);
	void ChangeVel(XType y,bool Additive);
	void HitPositionChange(XType y,bool Additive);
	void DrawBoard(CDC *dc,int x,int y,bool DrawAll);
	void Replay();
	bool IsCurPlrHuman() {return Rule->Players[Rule->nCurPlayer].bIsHuman;}
	void ReplayRatePlus();
	void ReplayRateMinus();
	// Returns true if GameStage is gsPlaying or gsReplaying
	bool gsPlayingShot() {return Rule->GameStage==gsPlaying || Rule->GameStage==gsReplaying;}
	void Invalidate() {bNeedsRedraw=true;}

	_CLW_HIDE
	void SetPaused(bool New) {Timer.Pause(New);}
	bool GetPaused() {return Timer.IsPaused();}
	EGameStage shGet() {return Rule->GameStage;}
	void shSet(EGameStage New) {Rule->GameStage=New;}
	_CLW_UNHIDE

	__declspec(property(get=GetPaused, put=SetPaused)) bool bGamePaused;
	__declspec(property(get=shGet, put=shSet)) EGameStage GameStage;
};
//////////////////////////////////////////////////////////////////////
// End of my code
//////////////////////////////////////////////////////////////////////

#endif // !defined(AFX_GAME_H__70431C42_7436_11D5_8548_444553540000__INCLUDED_)
