// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Carrom.h"

#include "MainFrm.h"
#include "About.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_PAINT()
	ON_COMMAND(ID_OPTIONS_FULLSCREEN, OnOptionsFullscreen)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_COMMAND(ID_GAME_PAUSE, OnGamePause)
	ON_COMMAND(ID_OPTIONS_FRAMERATE, OnOptionsFramerate)
	ON_COMMAND(ID_GAME_START, OnGameStart)
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
	ON_WM_SETFOCUS()
	ON_WM_ACTIVATE()
	//}}AFX_MSG_MAP
	ON_WM_ENTERMENULOOP()
	ON_WM_EXITMENULOOP()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	IsFullScreen=false;
	bShowFrameRate=false;
	dMouseAcc=0.5;
	ShowCursor(FALSE);
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO:	Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	cs.style &= ~(WS_MAXIMIZEBOX | WS_SIZEBOX);

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	//cs.lpszClass = AfxRegisterWndClass(0);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::SetFullScreen(bool FullScreen)
{
	static CRect WindowRect;
	static DWORD dwStyle;
	if (FullScreen==IsFullScreen) return;	// Already in specified mode
	if (FullScreen)
	{
		GetWindowRect(&WindowRect);
		DEVMODE DevMode;
		ZeroMemory(&DevMode,sizeof DevMode);
		DevMode.dmSize=sizeof DevMode;
		DevMode.dmBitsPerPel=m_nBitsPixel;
		DevMode.dmPelsWidth=640;
		DevMode.dmPelsHeight=480;
		DevMode.dmFields=DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
		if (ChangeDisplaySettings(&DevMode,CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
		{
			MessageBox("Could not change to fullscreen mode.",NULL,MB_OK | MB_ICONWARNING);
			return;
		}
		dwStyle=GetWindowLong(m_hWnd,GWL_STYLE);
		SetWindowLong(m_hWnd,GWL_STYLE,dwStyle & ~WS_CAPTION);
		SetWindowPos(&wndTopMost,0,0,640,480,SWP_SHOWWINDOW);
		SetMenu(NULL);
	}
	else 
	{
		ChangeDisplaySettings(NULL,0);
		SetWindowLong(m_hWnd,GWL_STYLE,dwStyle);
		SetWindowPos(&wndNoTopMost,WindowRect.left,WindowRect.top,
			WindowRect.right-WindowRect.left,WindowRect.bottom-WindowRect.top,
			SWP_NOREPOSITION | SWP_SHOWWINDOW);
		SetMenu(pMainMenu);
	}
	pMainMenu->CheckMenuItem(ID_OPTIONS_FULLSCREEN,FullScreen ? MF_CHECKED : MF_UNCHECKED);
	IsFullScreen=FullScreen;
}

BOOL CMainFrame::OnIdle(LONG lCount)
{
	// I am calling this from CarromApp::OnIdle
	if (Game.bGamePaused) return FALSE;	// To stop mouse tracking
	CPoint point;
	GetCursorPos(&point);
	ScreenToClient(&point);
	if (point.x!=319 || point.y!=239)
	{
		CPoint Centre(319,239);
		ClientToScreen(&Centre);
		SetCursorPos(Centre.x,Centre.y);
		//mouse_event(MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE, 0x7FFF, 0x7FFF, 0, 0);
		//while (PeekMessage(&msg,NULL,WM_MOUSEMOVE,WM_MOUSEMOVE,PM_REMOVE));
		OnMouseMotion((point.x-319)*dMouseAcc,(point.y-239)*dMouseAcc);
	}
	Invalidate(FALSE);
	if (Game.gsPlayingShot()) return TRUE;
	else return FALSE;
}

void CMainFrame::OnPaint() 
{
	static EGameStage PrevStage=gsPlaying;
	static bool FirstTime=true;
	static CDC dc;			// device context for offscreen bitmap
	static CBitmap bmpOffScr;
	static COLORREF clrBkColor=RGB(40,40,20);
	CPaintDC dcMain(this);	// device context for painting
	if (FirstTime)
	{
		FirstTime=false;

		// Creating the dc, etc.
		dc.CreateCompatibleDC(&dcMain);
		m_nBitsPixel=dc.GetDeviceCaps(BITSPIXEL);
		//bmpOffScr.CreateCompatibleBitmap(&dc,640,480);
		bmpOffScr.CreateBitmap(640,480,1,m_nBitsPixel,NULL);
		dc.SelectObject(&bmpOffScr);
	}
	if (PrevStage!=Game.GameStage)
	{	
		PrevStage=Game.GameStage;
		// Drawing the background
		CBrush brush;
		brush.CreateSolidBrush(clrBkColor);
		dc.FillRect(CRect(0,0,640,480),&brush);
		Game.DrawBoard(&dc,100,40,true);
	}
	else Game.DrawBoard(&dc,100,40,false);

	if (bShowFrameRate && !Game.bGamePaused)
	{
		static long int Calls=0,FRate=0,PrevCalls=Calls;
		static long int Ticks=GetTickCount();
		CString Text;
		Text.Format("Frame: %ld, Rate: %ld ",Calls++,FRate);
		dc.SetTextColor(RGB(255,255,255));
		//dc.SetBkMode(TRANSPARENT);
		dc.SetBkColor(clrBkColor);
		dc.TextOut(20,10,Text);
		if (GetTickCount()-Ticks>=1000)
		{
			FRate=Calls-PrevCalls;
			PrevCalls=Calls;
			Ticks=GetTickCount();
		}
	}

	dcMain.BitBlt(0,0,640,480,&dc,0,0,SRCCOPY);
}

void CMainFrame::OnOptionsFullscreen() 
{
	SetFullScreen(!IsFullScreen);
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	CRect Rect,ClientRect;
	GetWindowRect(&Rect);
	GetClientRect(&ClientRect);
	Rect.right=Rect.right+640-ClientRect.right;
	Rect.bottom=Rect.bottom+480-ClientRect.bottom;
	MoveWindow(&Rect,FALSE);
	pMainMenu=GetMenu();
	//OnOptionsFramerate();	// Turn on frame rate
	OnGamePause();
	return 0;
}

void CMainFrame::OnEnterMenuLoop(BOOL bIsTrackPopupMenu)
{
	TempPause(true);
	CFrameWnd::OnEnterMenuLoop(bIsTrackPopupMenu);
	ShowCursor(TRUE);
	if (IsFullScreen) SetMenu(pMainMenu);
}

void CMainFrame::OnExitMenuLoop(BOOL bIsTrackPopupMenu)
{
	TempPause(false);
	CFrameWnd::OnExitMenuLoop(bIsTrackPopupMenu);
	ShowCursor(FALSE);
	if (IsFullScreen) SetMenu(NULL);
}

void CMainFrame::OnDestroy() 
{
	CFrameWnd::OnDestroy();
	SetFullScreen(false);
}

void CMainFrame::OnGamePause() 
{
	Game.bGamePaused=!Game.bGamePaused;
	pMainMenu->CheckMenuItem(ID_GAME_PAUSE,Game.bGamePaused ? MF_CHECKED : MF_UNCHECKED);
	ShowCursor(Game.bGamePaused);
	Game.Timer.Pause(Game.bGamePaused);
}

void CMainFrame::OnOptionsFramerate() 
{
	bShowFrameRate=!bShowFrameRate;
	pMainMenu->CheckMenuItem(ID_OPTIONS_FRAMERATE,bShowFrameRate ? MF_CHECKED : MF_UNCHECKED);
}

void CMainFrame::OnGameStart()
{
	if (m_StartDialog.DoModal()==IDOK)
	{
		if (Game.bGamePaused) OnGamePause();
		CRule *Rule=new CRule1(Game.TheBoard);
		Game.NewGame(Rule);
	}
}

void CMainFrame::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
/*	// Code to support cheating
	static char CheatStr[]="ITSEASY";
	static int Pos=0;
	if (CheatStr[Pos]==(char)nChar)
	{
		if (CheatStr[++Pos]==0) bCheatEnabled=!bCheatEnabled;
	}
	else if (CheatStr[0]==(char)nChar) Pos=1;
	else Pos=0;*/

	// Usual codes
	KeysPressed.Add(nChar);
	switch (nChar)
	{
		case VK_SPACE:	// Execute tracked shot
			if (Game.bTrackLines && (Game.GameStage==gsArranging || Game.GameStage==gsAiming)) 
				Game.NextStage();
			break;
		case 'Z':		// Go to end of shot, now!
			if (Game.gsPlayingShot())
				Game.Timer.SetTime((float)Game.m_xShotTime);
			//	Game.Timer.SetRate(16);	// Animation effect
			break;
		case 'R':		// OnMouseMove has yet another behaviour for 'R'
			Game.Replay();
			break;
		case 'T':
		//	if (bCheatEnabled || Game.bTrackLines)
			Game.bTrackLines=!Game.bTrackLines;
			Game.Invalidate();
			break;
		case '<'+128:	// This is the best expression I found for key '<'!
			Game.ReplayRateMinus();
			break;
		case '>'+128:
			Game.ReplayRatePlus();
			break;
		default:
			if (nChar>='0' && nChar<='9')
			{
				int i, Num=((nChar-'0')+9)%10+1;
				dMouseAcc=0.5;
				for (i=0;i<Num;i++) dMouseAcc/=2;
			}
	}
	CFrameWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CMainFrame::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar=='D') Game.HitPositionChange(0,false);
	KeysPressed.Remove(nChar);
	CFrameWnd::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CMainFrame::OnMouseMotion(XType dx,XType dy)
{
	if (Game.IsCurPlrHuman())
	{
		if (Game.GameStage==gsArranging || Game.GameStage==gsAiming)
		{
			if (KeysPressed['S'])
			{
				Game.TheBoard->MoveStricker(dx,dy,true);
				Game.Invalidate();
			}
			else if (KeysPressed['A'])
				Game.AimAtMoveBy(dx,dy);
			else if (KeysPressed['D'])
				Game.HitPositionChange(dy/dMouseAcc/2,true);
			else if (Game.bTrackLines && KeysPressed['V'])
				Game.ChangeVel(dy,true);
			else if (Game.GameStage==gsArranging && KeysPressed['R'])
			{
				Game.TheBoard->ArrangeBall(dx,true);
				Game.Invalidate();
			}
			else if (Game.GameStage==gsAiming && KeysPressed['Q'])
				Game.MoveSpotted(0,dx,dy);
			else if (Game.GameStage==gsAiming && KeysPressed['W'])
				Game.MoveSpotted(1,dx,dy);
		}
	}
}

void CMainFrame::OnHelpAbout() 
{
	ShowCursor(TRUE);
	About AboutBox;
	AboutBox.DoModal();
	ShowCursor(FALSE);
}

void CMainFrame::OnSetFocus(CWnd* pOldWnd) 
{
	CFrameWnd::OnSetFocus(pOldWnd);
	TempPause(false);
}

void CMainFrame::TempPause(bool Pause)
{
	static int PauseCount=0;
	if (Pause)
	{
		if (!Game.bGamePaused) {PauseCount++; Game.bGamePaused=true;}
	}
	else
	{
		if (PauseCount>0) {Game.bGamePaused=false; PauseCount--;}
	}
}

void CMainFrame::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	CFrameWnd::OnActivate(nState, pWndOther, bMinimized);
	
	// TODO: Add your message handler code here
	if (nState==WA_INACTIVE) {
		CFrameWnd::OnKillFocus(pWndOther);
		bool AmActive=false;
		for (HWND TopWnd=pWndOther ? pWndOther->m_hWnd : ::GetTopWindow(NULL);
			TopWnd!=NULL && !AmActive;
			TopWnd=::GetWindow(TopWnd,GW_OWNER))
			if (TopWnd==m_hWnd) AmActive=true;
		if (!AmActive)
		{
			SetFullScreen(false);
			TempPause(true);
		}
		KeysPressed.Clear();
	}
}
